/* codec_raw.c
 * Copyright (C) 2002 QT4Linux and OpenQuicktime Teams
 *
 * This file is part of OpenQuicktime, a free QuickTime library.
 *
 * Based on QT4Linux by Adam Williams.
 *
 * OpenQuicktime is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.
 *
 * OpenQuicktime is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: codec_raw.c,v 1.7 2002/12/15 01:51:56 nj_humfrey Exp $
 */

#include <string.h>

#include "openquicktime/openquicktime.h"
#include "openquicktime/structs.h"
#include "openquicktime/plugin.h"

#include "codec_simple.h"


/* Some static global variables */
static int Initialised = 0;           /* Initialising counter */

static const char   Signature[5]  = "raw ";
static const char	*Name = "8-bit Offset Linear"; 
static const char   *Version  = "0.1"; 
static const char   *Copyright  = "2002 OpenQuicktime Team"; 
static const char   *ModuleAuthor  = "Nicholas Humfrey"; 
static const char   *CodecAuthor  = "Nicholas Humfrey"; 



static int
init_rawaudio(oqt_t *file, int track)
{
	oqt_audio_map_t *atrack = &file->atracks[track];
	// *** Check samples are 8-bit ***
	return ++Initialised;
}

static int
delete_rawaudio(oqt_t *file, int track)
{
	oqt_audio_map_t *atrack = &file->atracks[track];
	return --Initialised;
}

static
int frames_to_bytes_raw(
	oqt_t *file,
	int track,
	int frames)
{
	int channels = oqt_get_audio_channels(file, track);
	
	return frames*channels;
}


static int
decode_rawaudio(oqt_t *file, int track, 
	       oqt_uint32_t inputsize, 
	       oqt_uint8_t  *input,
	       oqt_uint32_t outputsize, 
	       oqt_uint8_t  *output)
{
	oqt_uint32_t i;
	
	// This only works for 8-bit audio....
	
	// Sanity check
	if (outputsize < inputsize)
		fprintf(stderr, "decode_rawaudio: Output buffer is not big enough (%d != %d).\n", inputsize, outputsize);
	
	for(i = 0; i < inputsize; i++)
	{
		output[i] = (oqt_uint8_t)input[i] - 0x80;
	}

	return inputsize;
}


static int
encode_rawaudio(oqt_t *file, int track,
	       oqt_uint32_t num_samples,
	       oqt_uint8_t *input,
	       oqt_uint32_t outputsize,
	       oqt_uint8_t *output)
{
	oqt_uint32_t i, inputsize = (oqt_get_audio_bits(file, track)/8) * 
			   num_samples * oqt_get_audio_channels(file, track);
	
	// This only works for 8-bit audio....
	
	// Sanity check
	if (outputsize < inputsize)
		fprintf(stderr, "encode_rawaudio: Output buffer is not big enough (%d != %d).\n", inputsize, outputsize);
	
	for(i = 0; i < inputsize; i++)
	{
		output[i] = (oqt_uint8_t)input[i] + 0x80;
	}

	return inputsize;
}

oqt_codec_info_t*
oqt_plugin_register_raw( void )
{
	oqt_audio_codec_t *codec;

	// Allocate memory for codec record
	codec = oqt_allocate_audio_codec();
	if (!codec) return NULL;
	
	// Store information about the codec
	strncpy(codec->info.fourcc, Signature, 4);
	codec->info.name = Name;
	codec->info.version = Version;
	codec->info.copyright = Copyright;
	codec->info.module_author = ModuleAuthor;
	codec->info.codec_author = CodecAuthor;
	codec->info.interface_age = OQT_INTERFACE_AGE;

	// Set the callbacks
	codec->init_codec = init_rawaudio;
	//codec->prepare_encoder = prepare_encode_rawaudio;
	codec->encode = encode_rawaudio;
	codec->frames_to_bytes = frames_to_bytes_raw;
	codec->decode = decode_rawaudio;
	codec->decode = decode_rawaudio;
	codec->delete_codec = delete_rawaudio;

	return &(codec->info);
}
