/* codec_simple.c
 * Copyright (C) 2002 QT4Linux and OpenQuicktime Teams
 *
 * This file is part of OpenQuicktime, a free QuickTime library.
 *
 * Based on QT4Linux by Adam Williams.
 *
 * OpenQuicktime is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.
 *
 * OpenQuicktime is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: codec_simple.c,v 1.7 2002/10/28 13:51:12 nj_humfrey Exp $
 */

#include <string.h>

#include "openquicktime/openquicktime.h"
#include "openquicktime/structs.h"
#include "openquicktime/plugin.h"

#include "codec_simple.h"

static const char   Signature[5]  = "_SPA"; // Standard Codec Signature
static const char	*Name = "Simple Audio Codec Loader"; 
static const char   *Version  = "0.1"; 
static const char   *Copyright  = "2002 OpenQuicktime Team"; 
static const char   *ModuleAuthor  = "Nicholas Humfrey"; 
static const char   *CodecAuthor  = ""; 


oqt_codec_info_t* oqt_plugin_register( void )
{
	oqt_audio_codec_t *simple=NULL;
	oqt_codec_info_t *ima4=NULL, *twos=NULL, *raw=NULL,
					 *alaw=NULL, *sowt=NULL, *ulaw=NULL,
					 *mace3=NULL, *mace6=NULL;
	
	// Create the Simple codec loader
	simple = oqt_allocate_audio_codec();
	if (!simple) return NULL;
	
	// Store information about the codec
	strncpy(simple->info.fourcc, Signature, 4);
	simple->info.name = Name;
	simple->info.version = Version;
	simple->info.copyright = Copyright;
	simple->info.module_author = ModuleAuthor;
	simple->info.codec_author = CodecAuthor;
	simple->info.interface_age = OQT_INTERFACE_AGE;
	simple->info.has_children = 1;
	
	
	// Load and register the raw codec
	raw = oqt_plugin_register_raw();
	if (!raw || oqt_register_codec(raw)) {
		fprintf(stderr, "Failed to register raw audio codec.\n");
	} else raw->parent = &simple->info;

	// Load and register the alaw codec
	alaw = oqt_plugin_register_alaw();
	if (!alaw || oqt_register_codec(alaw)) {
		fprintf(stderr, "Failed to register a-law codec.\n");
	} else alaw->parent = &simple->info;

	// Load and register the IMA4 codec
	ima4 = oqt_plugin_register_ima4();
	if (!ima4 || oqt_register_codec(ima4)) {
		fprintf(stderr, "Failed to register IMA4 codec.\n");
	} else ima4->parent = &simple->info;

	// Load and register the Mace 3:1 codec
	mace3 = oqt_plugin_register_mace3();
	if (!mace3 || oqt_register_codec(mace3)) {
		fprintf(stderr, "Failed to register MACE 3:1 codec.\n");
	} else mace3->parent = &simple->info;

	// Load and register the Mace 6:1 codec
	mace6 = oqt_plugin_register_mace6();
	if (!mace6 || oqt_register_codec(mace6)) {
		fprintf(stderr, "Failed to register MACE 6:1 codec.\n");
	} else mace6->parent = &simple->info;
	
	// Load and register the twos codec
	twos = oqt_plugin_register_twos();
	if (!twos || oqt_register_codec(twos)) {
		fprintf(stderr, "Failed to register twos codec.\n");
	} else twos->parent = &simple->info;
	
	// Load and register the sowt codec
	sowt = oqt_plugin_register_sowt();
	if (!sowt || oqt_register_codec(sowt)) {
		fprintf(stderr, "Failed to register sowt codec.\n");
	} else sowt->parent = &simple->info;
	
	// Load and register the ulaw codec
	ulaw = oqt_plugin_register_ulaw();
	if (!ulaw || oqt_register_codec(ulaw)) {
		fprintf(stderr, "Failed to register u-law codec.\n");
	} else ulaw->parent = &simple->info;


	return &simple->info;
}


