/* plugin.h
 * Copyright (C) 2001 QT4Linux and OpenQuicktime Teams
 *
 * This file is part of OpenQuicktime, a free QuickTime library.
 *
 * Based on QT4Linux by Adam Williams.
 *
 * OpenQuicktime is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.
 *
 * OpenQuicktime is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 

/*!
	@header OpenQuicktime Plugin
	OpenQuicktime aims to be a portable library for handling Apple's QuickTime (TM)
	popular media files on Unix-like environments.
	
	Visit the OpenQuicktime Homepage <A HREF="http://www.openquicktime.org">
	http://www.openquicktime.org</A> for more 
	information.
	
	This header file contains function declarations for use internally by plugins
*/

#ifndef OQT_PLUGIN_H__
#define OQT_PLUGIN_H__

#include "openquicktime/structs.h"

#ifdef __cplusplus
extern "C" {
#endif


/*!
  @function			oqt_register_codec
  @abstract			Add a codec to the registry
  @discussion		'info' should be created using
  					oqt_allocate_audio_codec or oqt_allocate_video_codec.
  					Should only be called by plugins that register more than
  					one codec. The primary codec in plugins is registered automatically.
  @param info		Pointer to a valid codec information record
  @result			Returns 0 if registering was successful
*/
OPENQUICKTIMELIB_API int OQT_STDCALL oqt_register_codec( oqt_codec_info_t* info );


/* In codec_stub.c */
/*!
  @function			oqt_allocate_audio_codec
  @abstract			Allocate memory for audio codec and set defaults
  @result			Returns pointer to the allocated memory
*/
OPENQUICKTIMELIB_API oqt_audio_codec_t* OQT_STDCALL oqt_allocate_audio_codec(void);

/*!
  @function			oqt_allocate_video_codec
  @abstract			Allocate memory for video codec and set defaults
  @result			Returns pointer to the allocated memory
*/
OPENQUICKTIMELIB_API oqt_video_codec_t* OQT_STDCALL oqt_allocate_video_codec(void);



/*
	This function should be defined (once) in each plugin.
	It is the entry point for the plugin loader
*/
extern oqt_codec_info_t* oqt_plugin_register( void );




#ifdef __cplusplus
}
#endif

#endif
