/* codec_stub.c
 * Copyright (C) 2002 QT4Linux and OpenQuicktime Teams
 *
 * This file is part of OpenQuicktime, a free QuickTime library.
 *
 * Based on QT4Linux by Adam Williams.
 *
 * OpenQuicktime is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.
 *
 * OpenQuicktime is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: codec_stub.c,v 1.17 2002/10/28 13:36:44 nj_humfrey Exp $
 */

#include "openquicktime/colormodels.h"
#include "openquicktime/structs.h"
#include "openquicktime/openquicktime.h"
#include "openquicktime/plugin.h"

#include "funcprotos.h"


// This doesn't do anything useful
// it is really only of interest for debugging
static int Initialised = 0;

static const char	Signature[5]  = "STUB"; 
static const char	*Name  = "Uninitialised Codec"; 
static const char   *Version  = "Unknown"; 
static const char   *Copyright  = "2002 QT4Linux and OpenQuicktime Teams"; 
static const char   *License  = "GNU Lesser General Public License"; 
static const char   *ModuleAuthor  = ""; 
static const char   *CodecAuthor  = ""; 



static int oqt_init_vcodec_stub(oqt_t *file, int track)
{
	fprintf(stderr,"oqt_init_vcodec_stub called [%d]\n", ++Initialised);
	return Initialised;
}

static int oqt_init_acodec_stub(oqt_t *file, int track)
{
	fprintf(stderr,"oqt_init_acodec_stub called [%d]\n", ++Initialised);
	return Initialised;
}


static int oqt_delete_vcodec_stub(oqt_t *file, int track)
{
	fprintf(stderr,"oqt_delete_vcodec_stub called [%d]\n", --Initialised);
	return Initialised;
}

static int oqt_delete_acodec_stub(oqt_t *file, int track)
{
	fprintf(stderr,"oqt_delete_acodec_stub called [%d]\n", --Initialised);
	return Initialised;
}



// It is valid for codecs not to implement this method
static int oqt_prepare_encoder_audio_stub(
			oqt_t *file,
			int track)
{
#ifdef DEBUG
	fprintf(stderr,"oqt_prepare_encoder_audio_stub called\n");
#endif
	return 0;
}


// It is valid for codecs not to implement this method
// Converts a number of frames of audio to its length in bytes.
// Audio codecs are only required to give this if it is possible to
// calculate - more complex codecs can rely on the STSZ table.
static int oqt_frames_to_bytes_stub(
			oqt_t *file,
			int track,
			int frames)
{
#ifdef DEBUG
	//fprintf(stderr,"oqt_frames_to_bytes_stub called\n");
#endif
	return 0;
}




static int oqt_reads_colormodel_stub(oqt_t *file, 
		int track,
		int colormodel)
{
#ifdef DEBUG
	fprintf(stderr,"oqt_reads_colormodel_stub called\n");
#endif
	return 0;
}

static int oqt_writes_colormodel_stub(oqt_t *file, 
		int track,
		int colormodel)
{
#ifdef DEBUG
	fprintf(stderr,"oqt_writes_colormodel_stub called\n");
#endif
	return 0;
}

static int oqt_preferred_colormodel_stub(oqt_t *file, 
		int track)
{
#ifdef DEBUG
	fprintf(stderr,"oqt_preferred_colormodel_stub called.\n");
#endif
	return BC_UNSUPPORTED;
}





// It is valid for codecs not to implement this method
static int oqt_set_param_stub (oqt_t *file, int track,
				     const char* param, const void* data)
{
#ifdef DEBUG
	fprintf(stderr,"oqt_set_param_stub called\n");
#endif
	return OQT_PARAMETER_UNKNOWN;
}

// It is valid for codecs not to implement this method
static int oqt_get_param_stub (oqt_t *file, int track,
				     const char* param, void* data)
{
#ifdef DEBUG
	fprintf(stderr,"oqt_get_param_stub called\n");
#endif
	return OQT_PARAMETER_UNKNOWN;
}


// It is valid for codecs not to implement this method
static const oqt_codec_param_t* oqt_list_params_stub(int *onum_params)
{
	// Default to no paramaters
	*onum_params = 0;
	return NULL;
}


OPENQUICKTIMELIB_API oqt_audio_codec_t* OQT_STDCALL oqt_allocate_audio_codec()
{
	oqt_audio_codec_t * codec;
	
/*#ifdef DEBUG
	fprintf(stderr, "Allocating memory for an audio codec.\n");
#endif*/

	// Allocate the memory for the codec record
	codec = calloc(1, sizeof(oqt_audio_codec_t));
	if(!codec) {
		fprintf(stderr, "Failed to allocate memory for oqt_audio_codec_t.\n");
		return NULL;
	}

	// Now set the defaults...
	memcpy(codec->info.fourcc, Signature, 4);

	codec->info.plugin_handle = NULL;
	codec->info.parent = NULL;
	codec->info.has_children = 0;
	codec->info.merit = OQT_NORMAL_MERIT;

	codec->info.name = Name;
	codec->info.type = OQT_CODEC_TYPE_AUDIO;
	codec->info.description = NULL;	// Description is not required
	codec->info.url = NULL;	// URL for *Module* not codec
							// and only set for code outside main source tree
	codec->info.version = Version;
	codec->info.copyright = Copyright;
	codec->info.license = License;
	codec->info.module_author = ModuleAuthor;
	codec->info.codec_author = CodecAuthor;
	codec->info.interface_age = OQT_INTERFACE_AGE;


	codec->init_codec = oqt_init_acodec_stub;
	codec->delete_codec = oqt_delete_acodec_stub;
	codec->list_params = oqt_list_params_stub;
	codec->decode = NULL;	// if set to NULL then it is assumed codec
							// does not support decoding
							
	codec->prepare_encoder = oqt_prepare_encoder_audio_stub;
	codec->frames_to_bytes = oqt_frames_to_bytes_stub;
	codec->encode = NULL;	// if set to NULL then it is assumed codec
							// does not support encoding
							
	codec->set_param = oqt_set_param_stub;
	codec->get_param = oqt_get_param_stub;
	
	return codec;
}



OPENQUICKTIMELIB_API oqt_video_codec_t* OQT_STDCALL oqt_allocate_video_codec()
{
	oqt_video_codec_t *codec;
	
/*#ifdef DEBUG
	fprintf(stderr, "Allocating memory for a video codec.\n");
#endif*/

	// Allocate the memory for the codec record
	codec = calloc(1, sizeof(oqt_video_codec_t));
	if(!codec) {
		fprintf(stderr, "Failed to allocate memory for oqt_video_codec_t.\n");
		return NULL;
	}

	// Now set the defaults...
	memcpy(codec->info.fourcc, Signature, 4);

	codec->info.plugin_handle = NULL;
	codec->info.parent = NULL;
	codec->info.has_children = 0;
	codec->info.merit = OQT_NORMAL_MERIT;
	
	codec->info.name = Name;
	codec->info.type = OQT_CODEC_TYPE_VIDEO;
	codec->info.description = NULL;	// Description is not required
	codec->info.url = NULL;	// URL for *Module* not codec
							// and only set for code outside main source tree
	codec->info.version = Version;
	codec->info.copyright = Copyright;
	codec->info.license = License;
	codec->info.module_author = ModuleAuthor;
	codec->info.codec_author = CodecAuthor;
	codec->info.interface_age = OQT_INTERFACE_AGE;


	codec->init_codec = oqt_init_vcodec_stub;
	codec->delete_codec = oqt_delete_vcodec_stub;
	codec->list_params = oqt_list_params_stub;
	codec->decode = NULL;	// if set to NULL then it is assumed codec
							// does not support decoding
	codec->encode = NULL;	// if set to NULL then it is assumed codec
							// does not support encoding
							
	codec->set_param = oqt_set_param_stub;
	codec->get_param = oqt_get_param_stub;
	codec->reads_colormodel = oqt_reads_colormodel_stub;
	codec->writes_colormodel = oqt_writes_colormodel_stub;
	codec->preferred_colormodel = oqt_preferred_colormodel_stub;
	
	return codec;
}

