/* codec_8BPS.c
 * Copyright (C) 2002 XAnim and OpenQuicktime Teams
 *
 * This file is part of OpenQuicktime, a free QuickTime library.
 *
 * Based on XAnim by Mark Podlipec.
 *
 * OpenQuicktime is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.
 *
 * OpenQuicktime is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: codec_8BPS.c,v 1.3 2002/12/15 01:52:00 nj_humfrey Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "openquicktime/openquicktime.h"
#include "openquicktime/colormodels.h"
#include "openquicktime/plugin.h"

#include "codec_simple.h"

static int          Initialised   = 0;
static const char   Signature[5]  = "8BPS";
static const char   *Name = "Planar RGB (Photoshop)";
static const char   *Version  = "0.1";
static const char   *Copyright  = "2002 OpenQuicktime Team";
static const char   *ModuleAuthor  = "Nicholas Humfrey";
static const char   *CodecAuthor  = "Mark Podlipec";
static const char   *Description  = "Only works for 24bit movies (Millions)";


typedef struct {
	int moof;
} Param;  



static int decode_8BPS(oqt_t *file,
		       int track,
		       oqt_uint32_t inputsize,
		       oqt_uint8_t *input,
		       oqt_uint8_t **output,
		       int color_model)
{
	Param *p = (Param*)(file->vtracks[track].codec_private);

	if(color_model == BC_NONE) {
		// drop frame
		return 0;
		
	} else if (color_model != BC_RGB888) {
		fprintf(stderr, "decode_8BPS: unsupported colour model: %d\n", color_model);
		return -1;
		
	} else {
		int width = oqt_get_video_width(file, track);
		int height = oqt_get_video_height(file, track);
		int x, y, len;
		oqt_uint8_t *r_lp, *g_lp, *b_lp;
		oqt_uint8_t *rp, *gp, *bp;
		
		
		// Line Pointers for each Plane 
		r_lp = input;
		g_lp = r_lp + (height << 1);
		b_lp = g_lp + (height << 1);
		
		/* Start of Red Rows */
		rp = b_lp + (height << 1);
		
		/* Calculate Size of Compressed Red Plane */
		gp = r_lp; len = 0;
		for(y = 0; y < height; y++) { len += ((gp[0] << 8) | gp[1]); gp += 2; }
		
		/* Start of Green Rows */
		gp = rp + len;
		
		/* Calculate Size of Compressed Green Plane */
		bp = g_lp; len = 0;
		for(y = 0; y < height; y++) { len += ((bp[0] << 8) | bp[1]); bp += 2; }
		
		/* Start of Blue Rows */
		bp = gp + len;
		
  
		for(y=0; y<height; y++) {
			oqt_uint8_t *row = output[y];
		
			/* Red PLANE */
			len = r_lp[ (y*2) ] << 8;  len |= r_lp[ (y*2) + 1 ];
			for(x=0; (x < width) && (len > 0);)
			{
				oqt_uint8_t code = *rp++; len--;
				if (code <= 127) { code++; len -= code;
					while(code--) { row[(x*3)+0] = *rp++; x++; } }
				else if (code > 128) { code = (oqt_int32_t)(257) - code;
					while(code--) { row[(x*3)+0] = *rp; x++; } rp++; len--; }
			}
			
			/* Green PLANE */
			len = g_lp[ (y*2) ] << 8;  len |= g_lp[ (y*2) + 1 ];
			for(x=0; (x < width) && (len > 0);)
			{
				oqt_uint8_t code = *gp++; len--;
				if (code <= 127) { code++; len -= code;
					while(code--) { row[(x*3)+1] = *gp++; x++; } }
				else if (code > 128) { code = (oqt_int32_t)(257) - code;
					while(code--) { row[(x*3)+1] = *gp; x++; } gp++; len--; }
			}
			
			/* Blue PLANE */
			len = b_lp[ (y*2) ] << 8;  len |= b_lp[ (y*2) + 1 ];
			for(x=0; (x < width) && (len > 0);)
			{
				oqt_uint8_t code = *bp++; len--;
				if (code <= 127) { code++; len -= code;
					while(code--) { row[(x*3)+2] = *bp++; x++; } }
				else if (code > 128) { code = (oqt_int32_t)(257) - code;
					while(code--) { row[(x*3)+2] = *bp; x++; } bp++; len--; }
			}
		}
		
	}
		
	return 0;
}

/* 8-bit:
		int width = oqt_get_video_width(file, track);
		int height = oqt_get_video_height(file, track);
		oqt_uint8_t *rp = input + (height << 1);
		int x, y;
		oqt_uint8_t *buf = malloc(10000);
		oqt_uint8_t *yp = buf;
		
		
		fprintf(stderr, "decode_8BPS: delta=%d\n", (height << 1));
		fprintf(stderr, "decode_8BPS: data size = %d).\n", inputsize);
		
		for(y=0; y<height; y++) {
		 	int len = input[ (y*2) ] << 8;
		 	len |= input[ (y*2) + 1 ];
		 	//fprintf(stderr, "y=%d len=%d  ", y, len);
		 	
		 	x = 0;
			while((x < width) && (len > 0))
    		{
				oqt_uint8_t code = *rp++; len--; // code
				if (code <= 127) {
					code++; len -= code; x += code;
					while(code--) { *yp++ = *rp++; }
				}
     			else if (code > 128) {
     				code = (oqt_uint32_t)(257) - code;
     				x += code; 
					while(code--) { *yp++ = *rp; }
					rp++; len--;
				}
   			}

		}
		
		for(y=0; y<height; y++) {
		
			memcpy(output[y], buf + (y*width), width);
		}
		
		
		free(buf);
*/

/*static int encode_8BPS(oqt_t *file,
				int track,
				oqt_uint8_t **input,
				oqt_uint32_t out_size,
				oqt_uint8_t *output,
				int *IsAKeyFrame,
				int color_model)
{
	Param *p = (Param*)(file->vtracks[track].codec_private);

	return 0;
}*/

static int reads_colormodel_8BPS(oqt_t *file,
		int track,
		int colormodel)
{
	int depth = oqt_get_video_depth(file, track);
	
	if (depth == 24) return ( colormodel == BC_RGB888 );
	else return 0;
}


/*
static int writes_colormodel_8BPS(oqt_t *file,
		int track,
		int colormodel)
{
	return ( colormodel == BC_RGB888 );
}
*/

static int preferred_colormodel_8BPS(oqt_t *file, 
		int track)
{
	int depth = oqt_get_video_depth(file, track);
	if (depth == 24) return BC_RGB888;
	else return BC_UNSUPPORTED;
}

static int init_8BPS(oqt_t *file, int track)
{
	oqt_video_map_t *vtrack = &file->vtracks[track];
	Param *p;
		
	/* Init params, should be called for every instance */
	vtrack->codec_private = malloc(sizeof(Param));
	if (!vtrack->codec_private) {
		fprintf(stderr, "init_8BPS: failed to allocate memory for codec.\n");
		return 0;
	}
	
	/* setup 8BPS */
	p = (Param*)vtrack->codec_private;
	memset(p, 0, sizeof(Param));


	return ++Initialised;
}

static int delete_8BPS(oqt_t *file, int track)
{
	oqt_video_map_t *vtrack = &file->vtracks[track];
	Param *p = (Param*)(vtrack->codec_private);

	if(p) {
		free(p);
		vtrack->codec_private=NULL;
	}

	return --Initialised;
}

oqt_codec_info_t*
oqt_simple_video_plugin_register_8BPS( oqt_video_codec_t *parent )
{
	oqt_video_codec_t *codec;
	
	codec = oqt_allocate_video_codec();
	if (!codec) return NULL;
	
	codec->info.parent = &(parent->info);
	parent->info.has_children = 1;
	
	codec->info.name = Name;
	memcpy( &codec->info.fourcc, Signature, 4);
	codec->info.version = Version;
	codec->info.copyright = Copyright;
	codec->info.description = Description;
	codec->info.module_author = ModuleAuthor;
	codec->info.codec_author = CodecAuthor;
	codec->info.interface_age = OQT_INTERFACE_AGE;
	
	codec->init_codec = init_8BPS;
	//codec->encode = encode_8BPS;
	codec->decode = decode_8BPS;
	codec->delete_codec = delete_8BPS;
	codec->reads_colormodel = reads_colormodel_8BPS;
	//codec->writes_colormodel = writes_colormodel_8BPS;
	codec->preferred_colormodel = preferred_colormodel_8BPS;
	
	if (oqt_register_codec(&codec->info)) {
		fprintf(stderr, "Failed to register 8BPS video codec\n");
		return NULL;
	}
	
	return &(codec->info);
}
